@@ -5,7 +5,7 @@ import numpy as np
 from bert4keras.layers import *
 from bert4keras.snippets import insert_arguments
 from bert4keras.snippets import delete_arguments
-from bert4keras.snippets import is_string
+from bert4keras.snippets import is_string, is_one_of
 from keras.models import Model
 import json
 
@@ -29,6 +29,7 @@ class Transformer(object):
         keep_tokens=None,  # 要保留的词ID列表
         compound_tokens=None,  # 扩展Embedding
         residual_attention_scores=False,  # Attention矩阵加残差
+        ignore_invalid_weights=False,  # 允许跳过不存在的权重
         layers=None,  # 外部传入的Keras层
         prefix=None,  # 层名前缀
         name=None,  # 模型名称
@@ -55,6 +56,7 @@ class Transformer(object):
         self.position_bias = None
         self.attention_scores = None
         self.residual_attention_scores = residual_attention_scores
+        self.ignore_invalid_weights = ignore_invalid_weights
         self.layers = {} if layers is None else layers
         self.prefix = prefix or ''
         self.name = name
@@ -282,8 +284,17 @@ class Transformer(object):
         weight_value_pairs = []
         for layer, variables in mapping.items():
             layer = self.layers[layer]
-            weights = layer.trainable_weights
-            values = [self.load_variable(checkpoint, v) for v in variables]
+            weights, values = [], []
+
+            for w, v in zip(layer.trainable_weights, variables):  # 允许跳过不存在的权重
+                try:
+                    values.append(self.load_variable(checkpoint, v))
+                    weights.append(w)
+                except Exception as e:
+                    if self.ignore_invalid_weights:
+                        print('%s, but ignored.' % e.message)
+                    else:
+                        raise e
 
             if isinstance(layer, MultiHeadAttention):
                 """如果key_size不等于head_size，则可以通过
@@ -298,15 +309,15 @@ class Transformer(object):
                 W = np.linalg.qr(np.random.randn(key_size, head_size))[0].T
                 if layer.attention_scale:
                     W = W * key_size**0.25 / head_size**0.25
-                for i in range(count):
-                    w, v = weights[i], values[i]
-                    w_shape, v_shape = K.int_shape(w), v.shape
-                    if w_shape[-1] != v_shape[-1]:
-                        pre_shape = w_shape[:-1]
-                        v = v.reshape(pre_shape + (heads, head_size))
-                        v = np.dot(v, W)
-                        v = v.reshape(pre_shape + (heads * key_size,))
-                        values[i] = v
+                for w, v in zip(weights, values):
+                    if is_one_of(w, layer.trainable_weights[:count]):
+                        w_shape, v_shape = K.int_shape(w), v.shape
+                        if w_shape[-1] != v_shape[-1]:
+                            pre_shape = w_shape[:-1]
+                            v = v.reshape(pre_shape + (heads, head_size))
+                            v = np.dot(v, W)
+                            v = v.reshape(pre_shape + (heads * key_size,))
+                            values[weights.index(w)] = v
 
             weight_value_pairs.extend(zip(weights, values))
 
@@ -2303,6 +2314,8 @@ def build_transformer_model(
     if is_string(model):
         model = model.lower()
         MODEL = models[model]
+        if model.startswith('t5.1.1'):
+            configs['version'] = 't5.1.1'
     else:
         MODEL = model
 
@@ -2318,9 +2331,6 @@ def build_transformer_model(
     elif application == 'unilm':
         MODEL = extend_with_unified_language_model(MODEL)
 
-    if model.startswith('t5.1.1'):
-        configs['version'] = 't5.1.1'
-
     transformer = MODEL(**configs)
     transformer.build(**configs)
 
