@@ -8,7 +8,8 @@ import re
 import sys
 from collections import defaultdict
 import json
-import keras
+import tensorflow as tf
+from bert4keras.backend import K, keras
 
 _open_ = open
 is_py2 = six.PY2
@@ -16,7 +17,7 @@ is_py2 = six.PY2
 if not is_py2:
     basestring = str
 
-    
+
 def to_array(*args):
     """批量转numpy的array
     """
@@ -156,7 +157,7 @@ def parallel_apply(
 
     in_queue, out_queue, seed_queue = Queue(max_queue_size), Queue(), Queue()
     if random_seeds is True:
-        random_seeds = np.random.randint(0, 2**32, workers)
+        random_seeds = [None] * workers
     elif random_seeds is None or random_seeds is False:
         random_seeds = []
     for seed in random_seeds:
@@ -235,6 +236,19 @@ def sequence_padding(inputs, length=None, padding=0, mode='post'):
     return np.array(outputs)
 
 
+def truncate_sequences(maxlen, index, *sequences):
+    """截断总长度至不超过maxlen
+    """
+    sequences = [s for s in sequences if s]
+    while True:
+        lengths = [len(s) for s in sequences]
+        if sum(lengths) > maxlen:
+            i = np.argmax(lengths)
+            sequences[i].pop(index)
+        else:
+            return sequences
+
+
 def text_segmentate(text, maxlen, seps='\n', strips=None):
     """将文本按照标点符号划分为若干个短句
     """
@@ -330,6 +344,45 @@ class DataGenerator(object):
             for d in self.__iter__(random):
                 yield d
 
+    def to_dataset(self, types, shapes, names=None, padded_batch=False):
+        """转为tf.data.Dataset格式
+        如果传入names的话，自动把数据包装成dict形式。
+        """
+        if names is None:
+
+            generator = self.forfit
+
+        else:
+
+            if is_string(names):
+                warps = lambda k, v: {k: v}
+            elif is_string(names[0]):
+                warps = lambda k, v: dict(zip(k, v))
+            else:
+                warps = lambda k, v: tuple(
+                    dict(zip(i, j)) for i, j in zip(k, v)
+                )
+
+            def generator():
+                for d in self.forfit():
+                    yield warps(names, d)
+
+            types = warps(names, types)
+            shapes = warps(names, shapes)
+
+        if padded_batch:
+            dataset = tf.data.Dataset.from_generator(
+                generator, output_types=types
+            )
+            dataset = dataset.padded_batch(self.batch_size, shapes)
+        else:
+            dataset = tf.data.Dataset.from_generator(
+                generator, output_types=types, output_shapes=shapes
+            )
+            dataset = dataset.batch(self.batch_size)
+
+        return dataset
+
 
 class ViterbiDecoder(object):
     """Viterbi解码算法基类
@@ -660,8 +713,6 @@ class WebServing(object):
     """
     def __init__(self, host='0.0.0.0', port=8000, server='paste'):
 
-        import tensorflow as tf
-        from bert4keras.backend import K
         import bottle
 
         self.host = host
